#include "file.h"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <algorithm>

int File::idx = 1;

File::File(const std::string& fname) : m_fname(fname)
{}

void File::get_data(std::map<int, drug>& map)
{
	std::ifstream in(m_fname, std::ios::binary);
	drug _drug;
	while (in.read(reinterpret_cast<char*>(&_drug), sizeof(drug))) {
		map.insert(std::make_pair(idx, _drug));
		++File::idx;
	}
}

void File::add_to_file()
{
	std::ofstream file(m_fname, std::ios::binary | std::ios::app);
	if (!file.is_open()) {
		std::cerr << "Error: file cannot be opened!" << std::endl;
		exit(EXIT_FAILURE);
	}
	bool end = 0;
	drug _drug;
	std::cout << "Enter the information of medicinal product" << std::endl;
	while (!end) {
		std::cout << "Enter pharmacy number, drug name, number of packages available at this pharmacy, price of one package, date of receipt at the pharmacy, storage time(in days)." << std::endl;
		std::cin >> _drug;
		file.write(reinterpret_cast<const char*>(&_drug), sizeof(drug));
		std::cout << "Do you want to continue typing(0) or complete(1)?" << std::endl;
		std::cin >> end;
	}
}


void File::delete_from_file(int p_idx)
{
	std::map<int, drug> map;
	get_data(map);
	map.erase(p_idx);
	std::ofstream out(m_fname + "_upd", std::ios::binary);
	for (const auto& elem : map)
		out.write(reinterpret_cast<const char*>(&elem.second), sizeof(drug));
}

void File::replace_in_file(int p_idx, const drug& _drug)
{
	std::map<int, drug> map;
	get_data(map);
	std::ofstream out(m_fname + "_upd", std::ios::binary);

	map[p_idx] = _drug;
	for (const auto& elem : map)
		out.write(reinterpret_cast<const char*>(&elem.second), sizeof(drug));
}

void File::transform_to_text()
{
	std::map<int, drug> map;
	get_data(map);
	std::ofstream out(m_fname + ".txt");
	out << "| ID |   Pharmacy number  |      Drug name     | Number of packages |        Price       |        Date        |     Storage time   |" << std::endl;
	size_t count = 0;
	for (const auto& elem : map) {
		count++;
		out << "|" << std::setw(4) << count << "|"
			<< std::setw(20) << elem.second.pharmacy << "|"
			<< std::setw(20) << elem.second.name << "|"
			<< std::setw(20) << elem.second.quantity << "|"
			<< std::setw(20) << elem.second.price << "|"
			<< std::setw(20) << elem.second.date << "|"
			<< std::setw(20) << elem.second.storage_time << "|"
			<< std::endl;
	}
}


void File::search_by_pharmacy(int _pharm)
{
	std::map<int, drug> map;

	drug _drug;
	std::ifstream file(m_fname, std::ios::binary);
	while (file.read(reinterpret_cast<char*>(&_drug), sizeof(drug))) {
		map.insert(std::make_pair(File::idx, _drug));
		++File::idx;
	}

	std::vector<drug> search;
	for (const auto& _drug : map) {
		if (_drug.second.pharmacy == _pharm) {
			search.push_back(_drug.second);
		}
	}
	std::ofstream out(std::to_string(_pharm), std::ios::binary);
	if (!out.is_open()) {
		std::cerr << "Error: File cannot be opened" << std::endl;
		exit(EXIT_FAILURE);
	}
	for (const auto& _drug : search) {
		out.write(reinterpret_cast<const char*>(&_drug), sizeof(drug));
	}
}


void File::search_by_name(const std::string& _name)
{
	std::map<int, drug> map;
	
	drug _drug;
	std::ifstream file(m_fname, std::ios::binary);
	while (file.read(reinterpret_cast<char*>(&_drug), sizeof(drug))) {
		map.insert(std::make_pair(File::idx, _drug));
		++File::idx;
	}

	std::vector<drug> search;
	for (const auto& _drug : map) {
		if (_drug.second.name == _name) {
			search.push_back(_drug.second);
		}
	}
	std::ofstream out(_name, std::ios::binary);
	if (!out.is_open()) {
		std::cerr << "Error: File cannot be opened" << std::endl;
		exit(EXIT_FAILURE);
	}
	for (const auto& _drug : search) {
		out.write(reinterpret_cast<const char*>(&_drug), sizeof(drug));
	}
}


void File::search_by_date(const std::string& _date)
{
	std::map<int, drug> map;
	
	drug _drug;
	std::ifstream file(m_fname, std::ios::binary);
	while (file.read(reinterpret_cast<char*>(&_drug), sizeof(drug))) {
		map.insert(std::make_pair(File::idx, _drug));
		++File::idx;
	}

	std::vector<drug> search;
	for (const auto& _drug : map) {
		if (_drug.second.date == _date) {
			search.push_back(_drug.second);
		}
	}
	std::ofstream out(_date, std::ios::binary);
	if (!out.is_open()) {
		std::cerr << "Error: File cannot be opened" << std::endl;
		exit(EXIT_FAILURE);
	}
	for (const auto& _drug : search) {
		out.write(reinterpret_cast<const char*>(&_drug), sizeof(drug));
	}
}
